@@ -109,7 +109,7 @@ class BW_Import {
 					return $post_id;
 				}
 				if (!$post_id) {
-					_e("Couldn't get post ID");
+					_e('Couldn&#8217;t get post ID');
 					echo '</li>';
 					break;
 				}
@@ -155,7 +155,7 @@ class BW_Import {
 			}
 			if ( $num_comments ) {
 				echo ' ';
-				printf( __ngettext('%s comment', '%s comments', $num_comments), $num_comments );
+				printf( _n('%s comment', '%s comments', $num_comments), $num_comments );
 			}
 			echo '</li>';
 			flush();
